/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.assignJan18;


import java.util.Random;

import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.TriggerAction;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;


/**
 * Data source.
 * Produces numbers in a sequence
 * @author ib27688
 *
 */
public class Source extends Component {
	
	Terminal out;
	Terminal ackOut;
	
	SourceAction srcAction;
	
	public Source(String name) {
		super(name, null);
		
		srcAction= new SourceAction("FifoSourceAction");
		addMember(srcAction);
		out= new OutputTerminal(FleetGlobal.OUT, this);
		ackOut= new InputTerminal(FleetGlobal.ACK_OUT, this);
		//attach the terminals
		srcAction.attach(out);
		srcAction.attach(ackOut);
	} // FifoSource
	
	public Terminal getTerminal(String name) {
		if (name.equals(FleetGlobal.OUT)) {
			return out;
		}
		else if (name.equals(FleetGlobal.ACK_OUT)) {
			return ackOut;
		}
		else {
			// unknown terminal
			return null;
		}
	} // getTerminal
	
	public String stateInfo() {
		
		String onOff = srcAction.isEnabled()
			? "Source is enabled." 
			: "Source is disabled."; 
		
		return "Source " + getName() + ", id= " + getID() + ",  has generated " 
		+ srcAction.next 
		+ " items. The next data item to be generated is "
		+ srcAction.next
		+ ". " 
		+ onOff;
	} // stateInfo
	
	private class SourceAction extends TriggerAction {

		int next=0;
		
		public SourceAction(String name) {
			super(name);
			// next data item to produce
			next= 0;

			//start enabled right away
			enabled= true;
		
			// how many data items to produce
			outputFunctions.put(FleetGlobal.OUT, new GenerateData());
			
		} // FifoSourceAction
		
		/** Generate new data items */
		private class GenerateData extends Function {

			private final int maxCount;
			private final boolean limitedRange;
			
			private Random rnd= new Random();
			private boolean displayGenerated;
			
			public GenerateData() {
				super("GenerateData");

				// what is the delay?
				Integer fDelay= globals.intValue(Assign1Defaults.SOURCE_DELAY);
				functionDelay= (fDelay != null)
					? new Delay(fDelay.intValue())
					: Assign1Defaults.SOURCE_DELAY_DEF;
	
				// log as you go?
				Boolean dGen= globals.booleanValue(Assign1Defaults.SOURCE_DISPLAY_GENERATED);	
				displayGenerated= (dGen != null)
					? dGen
					: Assign1Defaults.SOURCE_DISPLAY_GENERATED_DEF;
				
				// is there a MAX count
				Integer cMax= globals.intValue(Assign1Defaults.SOURCE_MAX_GENERATED);
				if (cMax != null) {
					maxCount= cMax;
					limitedRange= true;
				}
				else {
					maxCount= Integer.MAX_VALUE;
					limitedRange= false;
				}		
				if (maxCount < 0) {
					enabled= false;
				}
				
			} // GenerateData
			
			public Object value() {
				int current= next;
				// is this the last item to be generated?
				if (limitedRange && current >= maxCount) {
					// turn off
					enabled= false;
				}
				next++;

				Payload p= new Payload();
				Data d= new Data(current);
				p.data= d;
				p.source= 0;
				p.destination= rnd.nextInt(32);
				if (displayGenerated) {
					director.info("Source " 
							+ Source.this.getName() 
							+ ", ID= "
							+ Source.this.getID()
							+ ", generated payload "
							+ p);
				}
				
				
				return p;	
			} // value
		} // GenerateData


	} // SourceAction
	
} // FifoSource



